/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.application.impl;

import java.util.List;

import org.tinygroup.appconfig.AppConfigManager;
import org.tinygroup.application.Application;
import org.tinygroup.application.ApplicationProcessor;
import org.tinygroup.context.Context;
import org.tinygroup.context.impl.ContextImpl;
import org.tinygroup.fileresolver.FullContextFileRepository;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.order.processor.OrderProcessor;
import org.tinygroup.springutil.SpringUtil;
import org.tinygroup.vfs.FileObject;
import org.tinygroup.vfs.VFS;
import org.tinygroup.xmlparser.node.XmlNode;

/**
 * 默认的应用执行器
 * 
 * @author luoguo
 * 
 */
public class ApplicationDefault implements Application {
	private static final String WEBROOT = "TINY_WEBROOT";
	private static final String FILE_PROCESSOR_ORDER = "application.order.xml";
	private static Logger logger = LoggerFactory.getLogger(Application.class);
	private boolean isOrder;
	// 保存应用程序的坏境变量
	private Context context = new ContextImpl();
	// 应用处理器列表
	private List<ApplicationProcessor> applicationProcessors;
	// 配置信息管理
	private AppConfigManager appConfigManager;
	// 全路径文件注册对象
	FullContextFileRepository fullContextFileRepository;
	private XmlNode applicationConfig;

	public FullContextFileRepository getFullContextFileRepository() {
		return fullContextFileRepository;
	}

	public void setFullContextFileRepository(
			FullContextFileRepository fullContextFileRepository) {
		this.fullContextFileRepository = fullContextFileRepository;
	}

	public ApplicationDefault() {
	}

	public boolean isOrder() {
		return isOrder;
	}

	public void setOrder(boolean isOrder) {
		this.isOrder = isOrder;
	}

	/**
	 * 启动应用程序
	 */
	public void start() {
		String webRoot = context.get(WEBROOT);
		if (webRoot != null) {
			fullContextFileRepository.addSearchPath(webRoot);
		}
		if (applicationProcessors != null) {
			if (isOrder) {
				// 对ApplicationProcessor列表进行排序
				OrderProcessor<ApplicationProcessor> orderProcessor = SpringUtil
						.getBean(OrderProcessor.ORDER_NAME);
				String jarPath = getClass().getProtectionDomain()
						.getCodeSource().getLocation().getFile();
				FileObject fileObject = VFS.resolveFile(jarPath);
				orderProcessor.loadOrderFile(fileObject
						.getChild(FILE_PROCESSOR_ORDER));
				applicationProcessors = orderProcessor.orderList(
						"applicationProcessor", applicationProcessors);
			}
			for (ApplicationProcessor applicationProcessor : applicationProcessors) {
				applicationProcessor.setApplication(this);
				logger.logMessage(LogLevel.INFO, "应用处理器{}正在启动...",
						applicationProcessor.getClass());
				applicationProcessor.start();
				logger.logMessage(LogLevel.INFO, "应用处理器{}启动完毕。",
						applicationProcessor.getClass());
			}
		}
	}

	/**
	 * 关闭应用程序
	 */
	public void stop() {
		for (ApplicationProcessor processorLoader : applicationProcessors) {
			logger.logMessage(LogLevel.INFO, "应用处理器{}正在停止...",
					processorLoader.getClass());
			processorLoader.stop();
			logger.logMessage(LogLevel.INFO, "应用处理器{}停止完毕。",
					processorLoader.getClass());
		}
	}

	public Context getContext() {
		return context;
	}

	public void setApplicationProcessors(
			List<ApplicationProcessor> applicationProcessors) {
		this.applicationProcessors = applicationProcessors;
		appConfigManager.subscribeConfig(this);
		for (ApplicationProcessor processor : applicationProcessors) {
			appConfigManager.subscribeConfig(processor);
		}

	}

	public void setAppConfigManager(AppConfigManager appConfigManager) {
		this.appConfigManager = appConfigManager;

	}

	public String getNodeName() {
		return "application-properties";
	}

	public void setConfiguration(XmlNode xmlNode) {
		applicationConfig = xmlNode;
		if (applicationConfig != null) {
			List<XmlNode> subNodes = applicationConfig.getSubNodes("property");
			if (subNodes != null) {
				for (XmlNode property : subNodes) {
					String name = property.getAttribute("name");
					String value = property.getAttribute("value");
					if (name != null && value != null) {
						context.put(name, value);
						logger.logMessage(LogLevel.INFO,
								"从配置文件加载应用参数<{}>=<{}>。", name, value);
					} else {
						logger.logMessage(LogLevel.ERROR, "应用参数<{}>=<{}>设置失败！",
								name, value);
					}
				}
			}
		}
	}

	public XmlNode getConfiguration() {
		return applicationConfig;
	}

}
